########################################################################
# directory.pl -- routines for manipulating directories
########################################################################


####################################################################### 
# directory_exists -- attempts to determine if the directory exists or
#                     not.  If the directory does NOT exist, it attempts
#                     to create it.
#
#                     Regardless, this routine will not return TRUE
#                     unless the specified directory exists and is
#                     writeable by the real id.
#
# Note: This routine takes a MASK argument, but currently doesn't use
#       it.  Instead, directories created are given 0775 permissions.
#######################################################################
sub directory_exists {
    local($dir, $MASK) = @_ ;
    local($tmpdir)       = "" ;
    local(@path)         = () ;
    local($i)            = 0  ;
    local($j)            = 0  ;

    if ($dir !~ m%^/%) {
        print STDERR "Path is relative: $dir\n" if ($DEBUG) ;
        $tmpdir = `pwd` ;  # NOT portable on all machines!!
        chop($tmpdir);
        $dir = "$tmpdir/$dir" ;
    }

    @path = split('/', $dir) ;
    shift @path ;

    for ($i=@path; $i > 0; $i--) {
         $tmpdir = "" ;
         for ($j = 0 ; $j < $i; $j++) {
              $tmpdir .= "/" . $path[$j] ;
         }
         print STDERR "Testing: $tmpdir\n" if ($DEBUG) ;
         if (-e $tmpdir) {
             if ((-d $tmpdir) && (-R $tmpdir) && (-X $tmpdir)) {
                 if (! -W $tmpdir) {
                     print STDERR "$tmpdir not writeable!\n" if ($DEBUG);
                     return 0 ;
                 }
                 if ($i == scalar(@path)) {
                     print STDERR "Found it!\n" if ($DEBUG);
                     return 1 ;
                 }
                 while ($i != scalar(@path)) {
                     $tmpdir .= "/$path[$i]" ;
                     $! = 0 ;
                     print STDERR "Trying to make $tmpdir\n" if ($DEBUG);
                     mkdir ($tmpdir, 0775) ;  # 0775 should be $MASK
                     if ($!) {
                         print STDERR "Failed making $tmpdir\n" if ($DEBUG);
                          return 0 ;
                     }
                     $i++ ;
                 }
                 return 1 ;
             }
             else {
                 print STDERR "$tmpdir is not a directory!\n" if ($DEBUG);
             }
         }
         else {
             print STDERR "$tmpdir does not exist!\n" if ($DEBUG);
         }
    }
    print STDERR "Impossible to create $dir!\n" if ($DEBUG);
    return 0 ;
}

1;

