###############################################################################
# flows.pl -- interface to TextFlow structures
###############################################################################


###################################################
# get_flow_tag -- returns the tag name for a flow
###################################################
sub get_flow_tag {
    package MIFSTRUCT ;
    local(*flow) = @_ ;
    return &first_match(*flow, "<TFTag\\s+`(\\w+)'\\s*>");
}


###################################################
# get_flows_with_tag(tag) -- returns a list of flows with specified tag name
###################################################
sub get_flows_with_tag {
    package MIFSTRUCT ;
    local($tag) = @_ ;
    local(@flows) ;
    local(@flow) ;

    foreach (@TextFlows) {
        eval '@flow=@' . $_ ;
        push(@flows, $_) if (&get_flow_tag(*flow) eq $tag) ;
    }
    return @flows if (@flows);
    return "" ;
}


###################################################
# get_body_textflow -- returns textflow of Body pages (somewhat inefficient)
###################################################
sub get_body_textflow {
    package MIFSTRUCT ;
    local($flow) = "" ;
    local(@IDs)  = () ;
  
    foreach (&get_pages_of_type("Body")) {
        @IDs = &get_textrect_IDs($_) ;
        return &get_flow_with_textrect_ID($IDs[0]);
    }
}

###################################################
# get_flow_with_textrect_ID($ID) -- returns flow with specified
#                                   TextRectID from TextFlows
###################################################
sub get_flow_with_textrect_ID {
    package MIFSTRUCT ;
    local($ID) = @_ ;
    local(@flow) ;
    local(@para) ;
    local($para) ;
    local(@line) ;
    local($paraline) ;
    local($pattern);
  
# ALGORITHM:
#    Loop through ParaLines
#       Get ParaLine with specified TextRectID
#          Loop through Paras
#             Get Para with ParaLine
#                Loop through TextFlows
#                   Get TextFlow with Para

    foreach $paraline (@ParaLines) {
       if (&first_match($paraline, "<TextRectID\\s+($ID)\\s*>")) {
          foreach $para (@Paras) {
             if (&first_match($para, "^$paraline")) {
                foreach $flow (@TextFlows) {
                   if (&first_match($flow, "^$para")) {
                      return $flow ;
                   }
                }
             }
          }
       }
    }
    return "" ;
}



sub remove_textflow_from_top {
    local($textflow) = @_ ;
    local($i) = 0 ;

    return if (! defined(@TOP));
    for($i=0; $i< @TOP; $i++) {
       if ($TOP[$i] =~ /$textflow/) {
           splice(@TOP, $i, 1) ;
           return ;
       }
    }
}


sub get_text_in_flow {
    package MIFSTRUCT ;
    local($flowname) = @_ ;
    local(@flow)           = () ;
    local(@paras)          = () ;
    local(@para)           = () ;
    local($flowtext)       = "" ;
    local($paratext)       = "" ;
    local($sidewidth)      = 0 ;
    local($sidegap)        = 0 ;
    local($sidemargin)     = 0;
    local($sideheadmargin) = "" ;

    @flow = &get_array($flowname);

    $sidewidth = &first_match(*flow, "SideheadWidth\\s+([^\\s]+)\"");
    $sidegap = &first_match(*flow, "SideheadGap\\s+([^\\s]+)\"");
    $sidemargin = ($sidewidth + $sidegap) if ($sidewidth && $sidegap) ;

    @paras = &all_matches(*flow, "^Para") ;

    foreach (@paras) {
        @para = &get_array($_);
        ($paratext, $cont) = &get_text_in_para(*para, 0) ;
        $flowtext .= "\n$paratext" if ($paratext =~ /\w/);
    }
    return $flowtext if ($flowtext) ;
}



1;
