###############################################################################
# frames.pl -- interface to Frames (that is, anchored frames in MIF)
###############################################################################


########################################
# get_frame_from_id 
########################################
sub get_frame_from_id {
    package MIFSTRUCT ;
    local($ID) = @_ ;
    local(@frame);
    local($pattern);
    local($framename) = "";

    foreach (@Frames) {
       @frame = &get_array($_);
       $pattern = "^\\s*<ID\\s+($ID)\\s*>" ;
       $framename = &first_match(*frame, $pattern);
       if ($framename) {
          return $_ ;
       }
    }
    return "" ;
}



sub get_frame_dimensions {
    package MIFSTRUCT ;
    local($framename) = @_ ;
    local($frame);

    @frame = &get_array($framename) ;
    foreach (@frame) {
       if (/BRect\s+[\d\.]+"\s[\d\.]+"\s([\d\.]+)"\s([\d\.]+)\s>/) {
           return ($1, $2);
       }
       if (/ShapeRect\s+[\d\.]+"\s[\d\.]+"\s([\d\.]+)"\s([\d\.]+)\s>/) {
           return ($1, $2);
       }
    }
}



sub frame2mif {
    package MIFSTRUCT ;
    local($file,$framename) = @_ ;
    local(@frame)           = ();
    local($W, $H)           = (0.0, 0.0);
    local($PW, $PH)         = (0.0, 0.0);
    local($ID)              = "" ;
    local($units)           = "";
    local($i, $j)           = (0,0) ;
    local($textrectID)      = "" ;
    local(@flows)           = () ;
    local(%callbacks)       = () ;
    local($oldvalue)        = "" ; # for saving/restoring DPageRounding
   
    @frame= &get_array($framename) ;
    for ($i=0; $i < @frame; $i++) {
       $frame[$i] =~ s/<FrameType[^>]+>/<FrameType NotAnchored >/ ;
       splice (@frame, $i, 1) if ($frame[$i] =~ /<NSOffset/) ;
       splice (@frame, $i, 1) if ($frame[$i] =~ /<BLOffset/) ;
       splice (@frame, $i, 1) if ($frame[$i] =~ /<AnchorAlign/) ;

       #if ($frame[$i] =~ /^\s*<BRect([^>]+)>/) {
           #if (@dimensions = split($;, &get_dim($1, 4))) {
              #$W = $dimensions[2] ;
              #$H = $dimensions[3] ;
           #}
       #}
       #if ($frame[$i] =~ /^\s*<ShapeRect([^>]+)>/) {
           #if (@dimensions = split($;, &get_dim($1, 4))) {
              #$W = $dimensions[2] ;
              #$H = $dimensions[3] ;
           #}
       #}
       

if ($frame[$i]=~ /^\s*<BRect\s+[\d\.]+" [\d\.]+" ([\d\.]+)" ([\d\.]+)"/){
    $W = $1 ;
    $H = $2 ;
    $frame[$i] =~ s/^\s*<BRect\s+[\d\.]+" [\d\.]+"/  <BRect  0.0\" 0.0\"/;
}
if ($frame[$i]=~/^\s*<ShapeRect\s+[\d\.]+" [\d\.]+" ([\d\.]+)" ([\d\.]+)"/){
    $W = $1 ;
    $H = $2 ;
  $frame[$i] =~ s/^\s*<ShapeRect\s+[\d\.]+" [\d\.]+"/  <ShapeRect  0.0\" 0.0\"/;
}

       if ($frame[$i] =~ /<\s*TextRect\s+<ID\s+(\d+)\s*>/) {
          $ID = $1 ;
          $textrectID = &get_flow_with_textrect_ID($ID);
          &Out(2, "Frame contains ID: $ID in $textrectID");
          push(@flows, $textrectID) if ($textrectID);
       }
    }
 
    if (!$W || !$H) {
         print "Hmmm, dimensions are missing!  Skipping frame...";
         return "" ;
    }

    $callbacks{"AFrames"}  = "skip" ;   # HACK!!! THIS IS A SEVERE LIMITATION!!
    $callbacks{"TextFlow"} = "skip" ;   # HACK!!! THIS IS A SEVERE LIMITATION!!
    $callbacks{"Page"}     = "skip" ;             # This is NOT a limitation!!
    $callbacks{"Document"} = "replace_document" ; # This is to fix troublespots

    open (PS, ">$file")||die "Couldn't open $file for dumping graphic: $!";
    &print_token(*TOP, 0, "PS", *callbacks);
    &dump_graphic_to_mif(*frame, *flows, "PS");
    close PS || die "Couldn't close graphic file $file: $!"; 

    return ($W, $H);
}



sub dump_graphic_to_mif {
    local(*frame, *flows, $OUTFILE) = @_ ;
    local(@flow)                    = () ;
    local(@para)                    = () ;
    local(@paraline)                = () ;

    print $OUTFILE "<Verbose Yes>\n";
    print $OUTFILE "<Page\n";
    print $OUTFILE " <PageBackground `None'>\n";
    print $OUTFILE " <Frame\n";
    print $OUTFILE @frame ;
    print $OUTFILE " >\n" ;
    print $OUTFILE ">\n" ;
    $numflows = @flows ;
    &Out(2, "This file will have $numflows flows");
    foreach (@flows) {
       @flow = &get_array($_) ;
       print $OUTFILE "<TextFlow \n" ;
       foreach (@flow) {
           if (/^Para/) {
               @para= &get_array($_) ;
               print $OUTFILE " <Para \n" ;
               foreach (@para) {
                  if (/^ParaLine/) {
                      @paraline= &get_array($_) ;
                      print $OUTFILE "  <ParaLine \n" ;
                      foreach (@paraline) {
                         print $OUTFILE $_ ;
                      }
                      print $OUTFILE "  >\n" ;
                  }
                  else {
                      print $OUTFILE $_ ;
                  }
               }
               print $OUTFILE " > # end of Para\n" ;
           }
           else {
               print $OUTFILE $_ ;
           }
       }
       print $OUTFILE "> # end of TextFlow\n" ;
    }
    print $OUTFILE "# End of MIFFile";     # this is a hack (don't ask why)
}





sub replace_page {
    local($tokeninfo, $OUTFILE) = @_ ;
    local($token) = &get_name_from_info($tokeninfo);
 
    &Out(1, "Skipping page $token");
}




sub replace_textflow {
    local($tokeninfo, $OUTFILE) = @_ ;
    local($token) = &get_name_from_info($tokeninfo);
 
    &Out(1, "Skipping textflow $token");
}


sub replace_document {
    local($tokeninfo, $OUTFILE) = @_ ;
    local($POSTSCRIPT_PAGE_SIZE) = "8.5 11.0" ;

    &Out(1, "Replace Document section");

    $tokeninfo =~ s/DPageRounding ([^>]+)>/DPageRounding DontChangePageCount>/;
    $tokeninfo =~ s/DPageSize ([^>]+)>/DPageSize $POSTSCRIPT_PAGE_SIZE>/;

    print $OUTFILE $tokeninfo ;
}



sub frame2ps {
    package MIFSTRUCT ;
    local($tmpmif, $framename, $delay) = @_ ;
    local($print_settings)  = "$'INSTALLDIR/PrintToPS.doc" ;
    local($version)         = "" ;
    local($fmbatchpath)     = "" ;
    local($W)               = "" ;
    local($H)               = "" ;
    local($i, $j);

    #--------------------------------------------------------
    # Let's get version of Frame fmbatch needed
    # and the document with the print to file printer settings
    #--------------------------------------------------------

    ($version) = $TOP[0] =~ /<MIFFile\s+(\d.\d)/ ;   # HACK
    $fmbatchpath = $'Frame{$version} if ($'Frame{$version});
    if (!$fmbatchpath) {
        &Err("FrameMaker V$version is not on this system!") ;
        return "";
    }


    ($W, $H) = &frame2mif($tmpmif, $framename);

    &Out(0, "Opening $tmpmif\n");
    &open_fmbatch("| $fmbatchpath 2> /dev/null") ;
        &fmbatch("Open $tmpmif");
        &fmbatch("Open $print_settings");
        &fmbatch("Print $tmpmif $print_settings");
        &fmbatch("Quit $print_settings");
        &fmbatch("Quit $tmpmif");
    &close_fmbatch ;

    unlink $tmpmif ;
    print STDOUT "Sleeping..." ;
    sleep($delay);
    print STDOUT "snort...huh?...hmmm...whah...<yawn>...okay!" ;
    return ("${W}x$H") ;
}


1;
