###############################################################################
# generic.pl -- generic filter for use by other filters
###############################################################################

sub filter_para {
    package MIFSTRUCT ;
    local(*para, $ID, *callbacks)  = @_ ;
    local(@paralines)  = () ;  # holds array name of line of current paragraph
    local(@pgf)        = () ;  # holds each element of current paragraph
    local(@line)       = () ;  # holds each element of current line
    local($line)       = "" ;  # holds text on current line
    local(@matches)    = () ;  # ???
    local(@strings)    = () ;  # ???
    local($text)       = "" ;  # holds text of current paragraph
    local($pgfnum)     = "" ;  # holds final autonumber text of current para
    local($pattern)    = "" ;  # holds a regular expression
    local($tag)        = "" ;  # holds paragraph tag of current paragraph
    local($Findent)    = "" ;  # holds num spaces of First indent
    local($Lindent)    = "" ;  # holds num spaces of Left indent
    local($Numfont)    = "" ;  # holds Font of Autonumbering text
    local($Autonum)    = "" ;  # holds boolean for using paragraph's autonum
    local($Indent)     = "" ;  # holds spaces for current indent (Left or First)
    local($Numtabs)    = "" ;  # holds number of tabs in current para format
    local($NumStr)     = "" ;  # holds Autonumbering text of current paragraph
    local(@tabdefs)    = () ;  # holds elements of each tab 
    local(@tabstops)   = () ;  # holds location of each tab stop
    local(@ltabstops)  = () ;  # holds location of each tab stop local override
    local($tabpos)     = "" ;  # 
    local($tabalign)   = "" ;  # 
    local($tablead)    = "" ;  # 
    local($i)          = 0  ;  # counter
    local($j)          = 0  ;  # counter 
    local($k)          = 0  ;  # counter
    local($l)          = 0  ;  # counter 
    local($spaces)     = "" ;  # holds spaces for current tab indent
    local($PlStyle)    = "" ;  # 
    local($Sidemargin) = "" ;  # holds num inches in Sidemargin
    local($FFamily)    = "" ;  # holds font family of current active font
    local($DefFont)    = "" ;  # holds font family of default font for para
    local($ir)         = 20 ;  # holds num spaces to reduce left indent by
    local($cpi)        = 14 ;  # holds current chars per inch
    local($length)     = 80 ;  # holds longest margin line (unused)
    local($indent)     = 0  ;  # 
    local($array)      = "" ;
    local($pre)        = "" ;
    local($data)       = "" ;
    local($post)       = "" ;
    local(%PgfAttr)    = () ;
    local(%ParaAttr)   = () ;
    local($tabletitle) = "" ;
    local($tabletext)  = "" ;
    local($TableTitleFunc)   = "GetTableTitle" ;
    local($TableTextFunc)    = "GetTableText" ;
    local($OpenParaFunc)     = "" ;
    local($CloseParaFunc)    = "" ;
    local($OpenLineFunc)     = "" ;
    local($CloseLineFunc)    = "" ;
    local($FoundStringFunc)  = "GetString" ;
    local($FoundCharFunc)    = "GetChar" ;
    local($FoundTableFunc)   = "" ;


    $ir  = $IR if ( defined($IR) );
    $cpi = $CPI if ( defined($CPI) );
    $length = $LENGTH if ( defined($LENGTH) );
    
   
    #-----------------------------
    # Let's get Catalog based info
    #-----------------------------
    &GetPgfAttributeList(*para, *PgfAttr, *tabstops);
    $Findent = &GetParaAttribute(*PgfAttr, "PgfFIndent");
    $Lindent = &GetParaAttribute(*PgfAttr, "PgfLIndent");
    $Numfont = &GetParaAttribute(*PgfAttr, "PgfNumFont");
    $Numtabs = &GetParaAttribute(*PgfAttr, "PgfNumTabs");
    $Autonum = &GetParaAttribute(*PgfAttr, "PgfAutoNum");
    $FFamily = &GetParaAttribute(*PgfAttr, "FFamily");
    $PlStyle = &GetParaAttribute(*PgfAttr, "PgfPlacementStyle");

    #---------------------------------------------
    # Let's get local overrides (only one for now)
    #---------------------------------------------
    &GetParaAttributeList(*para, *ParaAttr, *ltabstops);
    $NumStr  = &GetParaAttribute(*ParaAttr, "PgfNumString");


    ######## Begin Frame 4.0 Specifics ##########

    $Sidemargin = $sidemargin if (($PlStyle eq "Normal") && ($sidemargin));

    ########  End Frame 4.0 Specifics  ##########



    #-----------------------------------
    # Get indentations and autonumbering
    #-----------------------------------

    $Findent  = (($Findent + $Sidemargin) * $cpi) ;
    $Findent += $ir if ($Findent > $ir);                # only clip if Findented
    $Lindent  = (($Lindent + $Sidemargin) * $cpi) ;
    $Lindent += $ir if ($Findent > $ir);                # only clip if Findented

    $Numfont = $FFamily if (!$Numfont);
    $pgfnum = &convert_entities_ascii($NumStr, $Numfont) if ($NumStr);


    #--------------------------
    # Now we set up our callbacks
    # We'll use scalars because 
    # Perl likes them better for direct calling...
    #---------------------------

    $OpenParaFunc    = $callbacks{'OpenPara'} ;
    $CloseParaFunc   = $callbacks{'ClosePara'} ;
    $OpenLineFunc    = $callbacks{'OpenLine'} ;
    $CloseLineFunc   = $callbacks{'CloseLine'} ;
    $FoundStringFunc = $callbacks{'FoundString'} if ($callbacks{'FoundString'});
    $FoundCharFunc   = $callbacks{'FoundChar'}   if ($callbacks{'FoundChar'});
    $FoundTableFunc  = $callbacks{'FoundTable'}  if ($callbacks{'FoundTable'});
    $TableTitleFunc  = $callbacks{'TableTitle'}  if ($callbacks{'TableTitle'});
    $TableTextFunc   = $callbacks{'TableText'}   if ($callbacks{'TableTitle'});



    &$OpenParaFunc(*PgfAttr) if ($OpenParaFunc);

    @paralines = &all_matches(*para, "^ParaLine");

    for ($i=0; $i < @paralines; $i++) {
        @line = &get_array($paralines[$i]);

        #---------------------------------
        # Check to see if we are still in right TextRect
        # This could be a hack since we are dependent upon the TextRects
        # incrementing all the way through.  But for now...
        #---------------------------------
        $id = &get_paraline_textrect(*line);
        if ($id && $ID) {
            next     if ($id < $ID) ;
            if ($id > $ID) {
                if (defined(%ONLY)) {
                    if (!$ONLY{$tag}) {
                        &Out(2, "Paragraph tagged: $tag  Skipping...");
                        return ("", 0) ;
                    }
                }
                return ($text, 0) 
            }
        }


        $line = "" ;
        &$OpenLineFunc(*PgfAttr, $i, *line) if ($OpenLineFunc);

        $Indent = ($i) ? $Lindent : $Findent ;
        $indent   = " " x $Indent ;
        $line .= $indent ;
        $line .= $pgfnum if ($Autonum && ($i == 0));
       
        
        #---------------------------------
        # Okay, now we are ready to process the current line
        #---------------------------------
        foreach (@line) {
           if (/<String `([^']+)'/) {
               &$FoundStringFunc(*PgfAttr,$1,$FFamily,*line) if($FoundStringFunc);
               next ;
           }
           if (/<Char\s+([\w]+)\s*>/) {
               &$FoundCharFunc(*PgfAttr,$1,$FFamily,*line) if($FoundCharFunc);
               next ;
           }
           if (/<ATbl\s+(\d+)\s*>/) {
               $tablename = &get_table_from_id($1) ;
               &$FoundTableFunc(*PgfAttr, *tablename, *line) if($FoundTableFunc) ;
               &$TableTitleFunc(*PgfAttr,$tablename,$ID,*line) if($TableTitleFunc);
               &$TableTextFunc(*PgfAttr,$tablename,$ID,*line) if($TableTextFunc);
               next ;
           }
           if (/<Font/) {
              &Out(2, "Found Font change");
              $* = 1 ;
              $fonttag = (/<FTag\s+`([^\']+)\'\s*>/) ? $1 : "" ;
              $FFamily = (/<FFamily\s+`([^\']+)\'\s*>/) ? $1 : $DefFont ;
              &Out(2, "Font Family is now $FFamily");
              $* = 0 ;
              next ;
           }
        }

        #---------------------------------
        # Done with the line, let's convert tabs
        #---------------------------------
        $k = index($line, "\\t") ;
        $j = 0 ;
        while ($k != -1) {
            if ($tabstops[$j]) {
                ($tabpos, $tabalign, $tablead) = split(/:/,$tabstops[$j]);
                if ($tabpos) {
                    $l  = (($tabpos + $Sidemargin) * $cpi) ;
                    $l += $ir if ($Findent > $ir);    # only clip if Findented
                    $spaces = ($l) ? ' ' x ($l - $k) : ' ' ;
                    $line =~ s/\\t/$spaces/ ;
                }
            }
            else {
              $line =~ s/\\t/ / ;
            }
            $j++ ;
            $k = index($line, "\\t") ;
        }

        #-----------------------------------
        # Add line's text to para then newline (unless last char is HardReturn)
        #-----------------------------------
        &$CloseLineFunc(*PgfAttr, *line) if ($CloseLineFunc);
        $text .= $line ;
    }
    &$CloseParaFunc(*PgfAttr, *text) if ($CloseParaFunc);
    return ($text, 0) ;
}


##############################################################################
#
# convert_entities()
#
#    Converts Frame graphic characters and special chars to ascii 
#    representations.  Takes care of quoted (markup) characters and
#    hexadecimal chars.  Mapping from Frame to ASCII occurs in fonts.pl
#
#############################################################################
sub convert_entities {
    package MIFSTRUCT ;
    local($_, $font) = @_ ;
    local($replacement)=  "" ;
    local($char) ;

    &Out(6, "Converting $font entities...");

    $* = 1 ;

    s/\\>/>/g;
    s/\\q/'/g;
    s/\\Q/`/g;
    s/\\\\/\\/g ;

    if ($font =~ /ingbat/) {
        $font = "dingbat" ;
    }
    elsif ($font =~ /ymbol/) {
        $font = "symbol" ;
    }
    else {
        $font = "standard" ;
    }

    while (/\\x([0-9a-f][0-9a-f]) /) {
        $char = $1 ;
        eval '$replacement = $ascii_' . $font . '_font{$char}' ;
        if (!$replacement) {
            print "\n" ;
            print "\n  Warning!  Deleting special char \\x$char ";
            print "\n  Font Family is: $font";
            print "\n  Text is: $_";
            print "\n" ;
            s/\\x[0-9a-f][0-9a-f] // ;
        }
        elsif ($replacement =~ /Reserved/) {
            &Err("Found Reserved $font char $char. Deleting...");
            s/\\x[0-9a-f][0-9a-f] // ;
        }
        else {
            &Out(2, "Replacing $font char \\x$char w/ $replacement");
            s/\\x[0-9a-f][0-9a-f] /$replacement/ ;
        }
    }

    $* = 0 ;
    return $_ ;
}


sub GetString {
    local(*PgfAttr, $str, $font, *line) = @_ ;
    
    $line .= &convert_entities($str, $font);
}

sub GetChar {
    local(*PgfAttr, $char, $font, *line) = @_ ;
    local($str) = "" ;

    $str = $ascii_characters{$char} ;
    $line .= &convert_entities($str, $font);
}


sub GetTableTitle {
    local(*PgfAttr, $tablename, $ID, *line) = @_ ;

    $line .= &get_table_title($tablename, $ID, 0);
    $line .= "\n" ; # could have been combined with above statement :)
}


sub GetTableText {
    local(*PgfAttr, $tablename, $ID, *line) = @_ ;
 
    $line .= &get_table_text($tablename, $ID, 1);
    $line .= "\n" ; # could have been combined with above statement :)
}


1;
