#############################################################################
# mmutil.pl -- Utility functions used by MifMucker Filters (.mmf files)
#
# Note these utilities assume the MIFSTRUCT package is in effect!!!!!!!!
#############################################################################

sub get_mmf_args {
    local(*argdefs, @args) = @_ ;
    local($i)              = 0 ;
    local(@ARGV)           = ();

    for ($i=0; $i < @args; $i++) {
        if (!($args[$i] =~ /^-/)) {
            push(@ARGV, $args[$i]);
            next ;
        }
        $args[$i] =~ s/^-// ;
        $found = 0 ;
        foreach (keys(%argdefs)) {
           if (/^$args[$i]/) {
              $found = 1 ;
              if ($argdefs{$_} eq "0") {
                  eval '$' . $_ . '= 1' ;
                  $found = 1 ;
                  last ;
              }
              if ($i == $#args) {
                  print "\nError: flag -$_ requires an argument.  Exiting...\n";
                  exit 1 ;
              }
              if ($argdefs{$_} eq "a") {
                  eval 'push(@' . $_ . ',$args[++$i])' ;
                  last ;
              }
              eval '$' . $_ . '="' . $args[++$i] . '"' ;
              last ;
           }
        }
        if (!$found) {
           print "\nError: Unknown flag -$args[$i].  Exiting...\n";
           exit 1 ;
        }
    }
    return @ARGV ;
}

###################################################
# first_match -- (*group, $pattern)
#    Searches through the specified array of tokens
#    looking for a pattern match.  If found, it returns
#    whatever matched (if you want the whole token make
#    sure your pattern sucks up the whole thing...
###################################################
sub first_match {
    local(*group, $pattern) = @_ ;
    local(@matches) ;

    foreach (@group) {
       if (@matches = /$pattern/) {
          return @matches if ($matches[0] ne "") ;
       }
    }
    return "" ;
}


###################################################
# get_elements_from_group -- (*group, $pattern)
#    Searches through the specified array of tokens
#    looking for pattern matches.  It returns a list
#    of all the elements of the array which matched
#    the pattern (the entire element is returned).
###################################################
sub all_matches {
    local(*group, $pattern) = @_ ;
    local(@matches) ;

    foreach (@group) {
       push(@matches, $_) if (/$pattern/) ;
    }
    return @matches if (@matches);
    return "" ;
}

sub splice_global {
    local (*group, $pos, $len, $element) = @_ ;
    if ($element) {
        return splice (@group, $pos, $len, $element) ;
    }
    return splice (@group, $pos, $len) ;
}


##########################################################################
# init_uniqueID -- initializes the uniqueID counter
##########################################################################
sub init_uniqueID {
    package Unique ;
    $UniqueID = 0  ;
}


##########################################################################
# create_new_name -- creates a new name based on a unique ID
##########################################################################
sub create_new_name {
    local($name)      = @_ ;

    $name .=  $Unique'UniqueID ;
    $Unique'UniqueID++ ;
    return $name ;
}


##########################################################################
# read_mapfiles -- reads global and local map files.  First tries to 
#                  read global map file for current filter ($'FILTER).
#                  If it exists, then tries to read local map file for
#                  current filter.
#
##########################################################################
sub read_mapfiles {
    local($mapfile)    = "$'FILTER.map" ;
    local($globalmap)  = "$'INSTALLDIR/$mapfile" ;
    local($localmap)   = "./$mapfile" ;

    return 0 if (!defined ($'FILTER));
    return 0 if (!defined ($'INSTALLDIR));

    ### LOOK FOR GLOBAL MAP FILE ###
    return 0 if ((!(-e $globalmap)) || (!(-r $globalmap))) ;
    require "$globalmap" ;
    &Out(1, "Read global map file");

    ### LOOK FOR LOCAL MAP FILE ###
    return 1 if ((!(-e $localmap)) || (!(-r $localmap))) ;
    require "$localmap" ;
    &Out(0, "Read local map file");
    return 2 ;
}


##########################################################################
# bynumber -- sort numerically
##########################################################################
sub bynumber {
    $a <=> $b ;
}

##########################################################################
# str2array -- convienence routine for returning the contents of an array
#              the string passed in is the name of the array.  Useful for
#              dynamically created arrays (means you don't need to eval).
##########################################################################
sub str2array {
    local(*name) = @_;
    return @name ;
}



##########################################################################
# Out -- Output a filter message based on verbosity level.
##########################################################################
sub Out {
    local($level, $message, $sameline) = @_ ;
    local($length)                     = 0  ;  
    local($prefix)                     = "" ;
    local($postfix)                    = "" ;
    local($filter)                     = "" ;

    #-------------------------------------------------------------------
    # This routine uses two "global" variables:
    #   $'DD       debugging detail level (only set in mmf territory)
    #   $'FILTER   name of current filter (might not be set in all cases)
    #
    # If DD isn't defined, then this routine is just being used for
    # general output (not in mmf sense) and so it simply outputs message
    #
    # If FILTER isn't defined, then this routine is just being used for
    # general controlled output, and so it simply outputs if DD is high
    # enough (DD > level).  
    #-------------------------------------------------------------------

    if (defined($'DD))  {  
        if ($'DD > $level) {
            if (defined($sameline) && $sameline) {  
                print STDOUT $message ;
                return ;
            }
            print "\n" ;
            if (defined($'FILTER)) { 
                $filter = $'FILTER ;
                $length = length($'FILTER) ;
                $prefix = ' ' x (9 - $length) ;
                $postfix = ' ' x $level ;
                print STDOUT "$prefix$filter : $postfix" ;
            }
            print STDOUT $message ;
        }
    }
    else {
       print STDOUT $message ;
    }
}
       
  
sub Err {
    local($message, $type) = @_ ;
  
    # Currently if $type is non-zero then program exits from die.  This
    # could be changed to allow various values for type to specify what
    # type of error message (FATAL, WARNING, and so on).
   
    print "\n" ;
    print STDERR "$'FILTER " if (defined($'FILTER));
    print STDERR "ERROR: $message" ;
    die "Exiting...\n" if (defined($type) && $type) ;
}


sub get_array {
    return &str2array(&get_name_from_info($_[0]));
}


sub get_name_from_info {
    local($info) = @_ ;
    local($name) = $info ;
    local($pre)  = "" ;
    local($data) = "" ;
    local($post) = "" ;
 
    ($name, $pre, $data, $post) = split(/$;/, $info) if ($info =~ /$;/) ;
    return $name ;
}

sub build {
    local (*array, $element) = @_ ;
    push(@array, $element) ;
}

sub NumElements {
    local (*array) = @_ ;
    return (defined(@array)) ? scalar(@array) : 0 ;
}

sub max3 {
    local($one, $two, $three) = @_ ;

    if ($one > $two) {
        return $one if ($one > $three) ;
    }
    else {
       return $two if ($two > $three) ;
    }
    return $three ;
}


sub print_token {
    local(*element, $indent, $OUTFILE, *callbacks) = @_ ;
    local($pre)              = ' ' x $indent if ($indent);
    local($token)            = "" ;
    local($data)             = "" ;
    local($post)             = "";
    local($array)            = "" ;

    foreach (@element) {
       if (/^[A-Za-z]/) {
          if (/$;/) {
             ($array, $pre, $data, $post) = split(/$;/, $_) ; 
          }
          else {
             &Out(1, "Element doesn't contain info: $_") ;
             $array = $_ ;
             $pre = "  <" ;
             $data = " \n" ;
             $post = " > # end of sometoken\n" ;
          }
          $token = $array ;
          $token =~ s/\d+$// ;
          if ($cb = $callbacks{$token}) {
              next if ($cb eq "skip") ;
              &Out(1, "Calling $cb for parsed $token");
              &$cb($_, $OUTFILE) ;
              next ;
          }
          &Out($indent+1, "Printing parsed token: $token") ;
          print $OUTFILE "$pre$token$data" ;
          &print_token($array, $indent+1, $OUTFILE, *callbacks) ;
          print $OUTFILE "$post" ;
       }
       elsif (/<([^\s]+)\s/) {
          $token = $1 ;
          if ($cb = $callbacks{$token}) {
              next if ($cb eq "skip") ;
              &Out(1, "Calling $cb for saved $token");
              &$cb($_, $OUTFILE) ;
              next ;
          }
          &Out($indent+1, "Printing saved token: $token");
          print $OUTFILE $_ ;
       }
    }
}
         
######################################################################
# mmcopy -- a generic copy routine with support for masks
########################################################################
sub mmcopy {
    local($src, $dest, $mask) = @_ ;
    local($test)        = "" ; 

    if (-e "$dest") {
        if (!$OVERWRITE) {
            if (! &YorN("Do you want to overwrite $dest", "Y")) {
                print "\nLeaving $dest..." ;
                return ;
            }
        }
        print "\nOverwriting $dest..." ;
    }
    else {
        print "\nWriting $dest..." ;
    }

    open (SRC, "$src") || die "Unable to open $src : $!\n" ;
    open (DES, ">$dest") || die "Unable to open $dest : $!\n" ;
    while (<SRC>) {
       print DES ;
    }
    close DES || die "Unable to close $src : $!\n" ;
    close SRC || die "Unable to close $dest : $!\n" ;

    eval '$test = chmod(' . $mask . ', $dest)';
    print "\nUnable to chmod $dest!" if (!$test);
}

1;
