###############################################################################
# pages.pl -- interface to Page structures
###############################################################################


###################################################
# get_page_type -- returns the Page's type
###################################################
sub get_page_type {
    package MIFSTRUCT ;
    local(*page) = @_ ;
    return &first_match(*page, "<PageType\\s+(\\w+)Page\\s*>");
}


###################################################
# get_page_tag -- returns the tag name for Master and Reference pages
###################################################
sub get_page_tag {
    package MIFSTRUCT ;
    local(*page) = @_ ;
    return &first_match(*page, "<PageTag\\s+`(\\w+)'\\s*>");
}


###################################################
# get_page_num -- returns the page number of Body pages
###################################################
sub get_page_num {
    package MIFSTRUCT ;
    local(*page) = @_ ;
    return &first_match(*page, "<PageNum\\s+`(\\d+)'\\s*>");
}



###################################################
# get_pages_of_type -- returns a list of the pages with specified type
###################################################
sub get_pages_of_type {
    package MIFSTRUCT ;
    local($pagetype) = @_ ;
    local(@pages) ;
    local(@page) ;

    foreach (@Pages) {
        eval '@page=@' . $_ ;
        push(@pages,$_) if (&get_page_type(*page) =~ /$pagetype/);
    }
    return @pages if (@pages);
    return "" ;
}



###################################################
# get_textrect_IDs -- returns a list of textrect IDs for a given page
###################################################
sub get_textrect_IDs {
    package MIFSTRUCT ;
    local (*page) = @_ ;
    local (@textrects);
    local(@IDs) ;

    @textrects = &all_matches(*page, "<TextRect");

    foreach (@textrects) {
       push(@IDs, $1) if (/<ID\s+(\d+)\s*>/) ;
    }
    return @IDs ;
}




###################################################
# get_text_in_ID -- print text inside given TextRect
###################################################
sub get_text_in_ID {
    package MIFSTRUCT ;
    local($ID)             = @_ ;
    local($rect)           = &get_flow_with_textrect_ID($ID);
    local(@flow)           = () ;
    local(@paras)          = () ;
    local(@para)           = () ;
    local($found)          = "" ;
    local($array)          = "" ;
    local($pre)            = "" ;
    local($data)           = "" ;
    local($post)           = "" ;
    local($pattern)        = "" ;
    local(@IDs)            = () ;
    local($num)            = "" ;
    local($recttext)       = "" ;
    local($paratext)       = "" ;
    local($sidewidth)      = 0 ;
    local($sidegap)        = 0 ;
    local($sidemargin)     = 0;
    local($sideheadmargin) = "" ;
    local($i)              = "" ;

    ($array, $pre, $data, $post) = split(/$;/, $rect) ;
    eval '@flow=@' . $array;

    $sidewidth = &first_match(*flow, "SideheadWidth\\s+([^\\s]+)\"");
    $sidegap = &first_match(*flow, "SideheadGap\\s+([^\\s]+)\"");
    $sidemargin = ($sidewidth + $sidegap) if ($sidewidth && $sidegap) ;

    @paras = &all_matches(*flow, "^Para") ;
    $num = @paras ;

    foreach (@paras) {
        ($array, $pre, $data, $post) = split(/$;/, $_);
        eval '@para=@' . $array ;
        if (!$found) {
           @IDs = &get_para_textrects(*para);
           $pattern = "^" . $ID . "$" ;
           $found = &first_match(*IDs, $pattern) ;
           next if (!$found);
        }
        ($paratext, $cont) = &get_text_in_para(*para, $ID) ;
        $recttext .= "\n$paratext" if ($paratext =~ /\w/);
        if (!$cont) {
           return $recttext ;
        }
    }
    return $recttext if ($recttext) ;
}


###################################################
# get_text_on_page -- prints the text on a given page
###################################################
sub get_text_on_page {
    package MIFSTRUCT ;
    local(*page)     = @_ ;
    local($idtext)   = "";
    local($pagetext) = "";
    local($array)    = "" ;

    $pagetext = "" ;
    foreach (&get_textrect_IDs(*page)) {
           $idtext = &get_text_in_ID($_);
           $pagetext .= $idtext if ($idtext);
    }
    return $pagetext ;
}


1;
