###############################################################################
# paras.pl -- interface to Pare structures
###############################################################################

########################################
# get_para_textrects -- returns an array of all specified textrects in para
########################################
sub get_para_textrects {
    local(*para) = @_ ;
    local(@paralines) ;
    local(@paraline) ;
    local(@IDs);
    local($ID);
    local($array, $pre, $data, $post) ;

    @paralines = &all_matches(*para, "^ParaLine");

    foreach (@paralines) {
        ($array, $pre, $data, $post) = split(/$;/, $_);
        eval '@paraline=@' . $array ;
        $ID = &get_paraline_textrect(*paraline) ;
        push(@IDs, $ID) if ($ID);
    }
    return @IDs ;
}


###########################################################################
# get_para_tag -- returns the paragraph tag name of a Paragraph
#
# ARGUMENTS -- pointer to a Para structure
#
# RETURNS -- string containing paragraph tag name of the Para structure
###########################################################################
sub get_para_tag {
    local(*para) = @_ ;
    local($tag) ;

    ($tag) = &first_match(*para, "PgfTag\\s*\\`([^\\']+)\\'");
    return $tag ;
}


######################################################################
# create_new_para -- creates a Para structure with of with given tag name 
#                    and text (also given AutoNumbering string).  The
#                    paragraph is given the name "ParaN#" where # is
#                    the next UniqueID number.  
#
# ARGUMENTS -- Paragraph tag name, AutoNumbering string, text
#
# CALLS -- &create_new_name() to get new array name 
#
# RETURNS -- The name of the newly created Para structure
######################################################################
sub create_new_para {
    package MIFSTRUCT ;
    local ($tag, $numstring, @content) = @_ ;
    local ($newname);
    local ($text) = "" ;

    $text .= " <Para \n" ;
    $text .= "  <PgfTag \`$tag\'>\n" if ($tag);
    $text .= "  <PgfNumString \`$numstring\'>\n" if ($numstring);

    foreach (@content) {
       $text .= "  <ParaLine \n" ;
       if (/^\s*</) {
           $text .= "$_\n";
       }
       else {
           $text .= "   <String \`$_\'>\n";
       }
       $text .= "   >\n" ;
    }
    $text .= "  > # end of Para\n";
 
    $newname = &get_Para($text);
    return $newname if ($newname);
    return "" ;
}



###########################################################
# get_text_in_para(*para, textrectID) -- prints the lines
#            of the paragraph, so long as they exist in the
#            specified TextRect
###########################################################
sub get_text_in_para {
    package MIFSTRUCT ;
    local(*para, $ID)  = @_ ;
    local(@paralines)  = () ;  # holds array name of line of current paragraph
    local(@pgf)        = () ;  # holds each element of current paragraph
    local(@line)       = () ;  # holds each element of current line
    local($line)       = "" ;  # holds text on current line
    local(@matches)    = () ;  # ???
    local(@strings)    = () ;  # ???
    local($text)       = "" ;  # holds text of current paragraph
    local($pgfnum)     = "" ;  # holds final autonumber text of current para
    local($pattern)    = "" ;  # holds a regular expression
    local($tag)        = "" ;  # holds paragraph tag of current paragraph
    local($Findent)    = "" ;  # holds num spaces of First indent
    local($Lindent)    = "" ;  # holds num spaces of Left indent
    local($Numfont)    = "" ;  # holds Font of Autonumbering text
    local($Autonum)    = "" ;  # holds boolean for using paragraph's autonum
    local($Indent)     = "" ;  # holds spaces for current indent (Left or First)
    local($Numtabs)    = "" ;  # holds number of tabs in current para format
    local($NumStr)     = "" ;  # holds Autonumbering text of current paragraph
    local(@tabdefs)    = () ;  # holds elements of each tab 
    local(@tabstops)   = () ;  # holds location of each tab stop
    local(@ltabstops)  = () ;  # holds location of each tab stop local override
    local($tabpos)     = "" ;  # 
    local($tabalign)   = "" ;  # 
    local($tablead)    = "" ;  # 
    local($i)          = 0  ;  # counter
    local($j)          = 0  ;  # counter 
    local($k)          = 0  ;  # counter
    local($l)          = 0  ;  # counter 
    local($spaces)     = "" ;  # holds spaces for current tab indent
    local($PlStyle)    = "" ;  # 
    local($Sidemargin) = "" ;  # holds num inches in Sidemargin
    local($FFamily)    = "" ;  # holds font family of current active font
    local($DefFont)    = "" ;  # holds font family of default font for para
    local($ir)         = 20 ;  # holds num spaces to reduce left indent by
    local($cpi)        = 14 ;  # holds current chars per inch
    local($length)     = 80 ;  # holds longest margin line (unused)
    local($indent)     = 0  ;  # 
    local($array)      = "" ;
    local($pre)        = "" ;
    local($data)       = "" ;
    local($post)       = "" ;
    local(%PgfAttr)    = () ;
    local(%ParaAttr)   = () ;

    $ir  = $IR if ( defined($IR) );
    $cpi = $CPI if ( defined($CPI) );
    $length = $LENGTH if ( defined($LENGTH) );
    
   
    #-----------------------------
    # Let's get Catalog based info
    #-----------------------------
    &GetPgfAttributeList(*para, *PgfAttr, *tabstops);
    $Findent = &GetParaAttribute(*PgfAttr, "PgfFIndent");
    $Lindent = &GetParaAttribute(*PgfAttr, "PgfLIndent");
    $Numfont = &GetParaAttribute(*PgfAttr, "PgfNumFont");
    $Numtabs = &GetParaAttribute(*PgfAttr, "PgfNumTabs");
    $Autonum = &GetParaAttribute(*PgfAttr, "PgfAutoNum");
    $FFamily = &GetParaAttribute(*PgfAttr, "FFamily");
    $PlStyle = &GetParaAttribute(*PgfAttr, "PgfPlacementStyle");

    #---------------------------------------------
    # Let's get local overrides (only one for now)
    #---------------------------------------------
    &GetParaAttributeList(*para, *ParaAttr, *ltabstops);
    $NumStr  = &GetParaAttribute(*ParaAttr, "PgfNumString");


    ######## Begin Frame 4.0 Specifics ##########

    $Sidemargin = $sidemargin if (($PlStyle eq "Normal") && ($sidemargin));

    ########  End Frame 4.0 Specifics  ##########



    #-----------------------------------
    # Get indentations and autonumbering
    #-----------------------------------

    $Findent  = (($Findent + $Sidemargin) * $cpi) ;
    $Findent += $ir if ($Findent > $ir);                # only clip if Findented
    $Lindent  = (($Lindent + $Sidemargin) * $cpi) ;
    $Lindent += $ir if ($Findent > $ir);                # only clip if Findented

    $Numfont = $FFamily if (!$Numfont);
    $pgfnum = &convert_entities_ascii($NumStr, $Numfont) if ($NumStr);





   #-----------------------------
   # Now the real work begins
   #-----------------------------

    @paralines = &all_matches(*para, "^ParaLine");

    for ($i=0; $i < @paralines; $i++) {
        ($array, $pre, $data, $post) = split(/$;/, $paralines[$i]) ;
        eval '@line = @' . $array ;
        $line = "" ;

        $Indent = ($i) ? $Lindent : $Findent ;
        $indent   = " " x $Indent ;
        $line .= $indent ;

        $line .= $pgfnum if ($Autonum && ($i == 0));
       
        #---------------------------------
        # Check to see if we are still in right TextRect
        # This could be a hack since we are dependent upon the TextRects
        # incrementing all the way through.  But for now...
        #---------------------------------
        $id = &get_paraline_textrect(*line);
        if ($id && $ID) {
            next     if ($id < $ID) ;
            if ($id > $ID) {
                if (defined(%ONLY)) {
                    if (!$ONLY{$tag}) {
                        &Out(2, "Paragraph tagged: $tag  Skipping...");
                        return ("", 0) ;
                    }
                }
                return ($text, 0) ;
            }
        }

        
        #---------------------------------
        # Okay, now we are ready to process the current line
        #---------------------------------
        foreach (@line) {
           if (/<String `([^']+)'/) {
               $line .= &convert_entities_ascii($1, $FFamily);
               next ;
           }
           if (/<Char\s+([\w]+)\s*>/) {
               $line .= &convert_entities_ascii($ascii_characters{$1},$FFamily);
               next ;
           }
           if (/<ATbl\s+(\d+)\s*>/) {
               $tablename = &get_table_from_id($1) ;
               if ($tablename) {
                  $line .= &get_table_title($tablename, $ID, 0) ;
                  $line .= "\n" ;
                  $line .= &get_table_text($tablename, $ID, 1);
                  $line .= "\n" ;
               }
               next ;
           }
           if (/<Font/) {
              &Out(5, "Found Font change");
              $* = 1 ;
              $fonttag = (/<FTag\s+`([^\']+)\'\s*>/) ? $1 : "" ;
              $FFamily = (/<FFamily\s+`([^\']+)\'\s*>/) ? $1 : $DefFont ;
              &Out(6, "Font Family is now $FFamily");
              $* = 0 ;
              next ;
           }
        }

        #---------------------------------
        # Done with the line, let's convert tabs
        #---------------------------------
        $k = index($line, "\\t") ;
        $j = 0 ;
        while ($k != -1) {
            if ($tabstops[$j]) {
                ($tabpos, $tabalign, $tablead) = split(/:/,$tabstops[$j]);
                if ($tabpos) {
                    $l  = (($tabpos + $Sidemargin) * $cpi) ;
                    $l += $ir if ($Findent > $ir);    # only clip if Findented
                    $spaces = ($l) ? ' ' x ($l - $k) : ' ' ;
                    $line =~ s/\\t/$spaces/ ;
                }
            }
            else {
              $line =~ s/\\t/ / ;
            }
            $j++ ;
            $k = index($line, "\\t") ;
        }

        #-----------------------------------
        # Add line's text to para then newline (unless last char is HardReturn)
        #-----------------------------------
        $text .= $line ;
        $text .= "\n" if ($text !~ /\n$/) ;
    }
    if (defined(%ONLY)) {
        if (!$ONLY{$tag}) {
            &Out(2, "Paragraph tagged: $tag  Skipping...");
            return ("", 1) ;
        }
    }
    return ($text, 1) ;
}



##############################################################################
#
# convert_entities_ascii()
#
#    Converts Frame graphic characters and special chars to ascii 
#    representations.  Takes care of quoted (markup) characters and
#    hexadecimal chars.  Mapping from Frame to ASCII occurs in fonts.pl
#
#############################################################################
sub convert_entities_ascii {
    package MIFSTRUCT ;
    local($_, $font) = @_ ;
    local($replacement)=  "" ;
    local($char) ;

    &Out(6, "Converting $font entities to ASCII entities");

    $* = 1 ;

    s/\\>/>/g;
    s/\\q/'/g;
    s/\\Q/`/g;
    s/\\\\/\\/g ;

    if ($font =~ /ingbat/) {
        $font = "dingbat" ;
    }
    elsif ($font =~ /ymbol/) {
        $font = "symbol" ;
    }
    else {
        $font = "standard" ;
    }

    while (/\\x([0-9a-f][0-9a-f]) /) {
        $char = $1 ;
        eval '$replacement = $ascii_' . $font . '_font{$char}' ;
        if (!$replacement) {
            print "\n" ;
            print "\n  Warning!  Deleting special char \\x$char ";
            print "\n  Font Family is: $font";
            print "\n  Text is: $_";
            print "\n" ;
            s/\\x[0-9a-f][0-9a-f] // ;
        }
        elsif ($replacement =~ /Reserved/) {
            &Err("Found Reserved $font char $char. Deleting...");
            s/\\x[0-9a-f][0-9a-f] // ;
        }
        else {
            &Out(2, "Replacing $font char \\x$char w/ $replacement");
            s/\\x[0-9a-f][0-9a-f] /$replacement/ ;
        }
    }

    $* = 0 ;
    return $_ ;
}


sub GetParaAttribute {
    local(*Attr, $att) = @_ ;
    return ($Attr{$att}) ? $Attr{$att} : "" ;
}



####################################################################
# GetPgfAttributeList -- stores PgfCatalog attributes of a paragraph
#
# Works by getting the Para out of the catalog and calling 
# GetParaAttributeList().
#
# See comments for GetParaAttributeList().
####################################################################
sub GetPgfAttributeList {
    local(*para, *Attr, *tabstops) = @_ ;
    local($tag)       = "" ;

    $tag = &get_para_tag(*para);
    foreach (@Pgfs) {
       @pgf = &get_array($_) ;
       if ($tag eq &get_para_tag(*pgf)) {
           &GetParaAttributeList(*pgf, *Attr, *tabstops) ;
           last ;
       }
    }
}



#######################################################################
# GetParaAttributeList -- stores attributes of a paragraph
#
# Pass in the name of a two arrays -- first is an associative array for
# storing attributes, the second is a normal array for holding tab info.
# Tab info is a string with the position in inches, alignment, and leading 
# specified separated by single colons.  So two tab stops at 3 and 5
# inches might look like this:
#
#   $tabstops[0] = "3.0:Left: "
#   $tabstops[1] = "5.0:Left: "
#
# THIS DOESN'T CURRENTLY GET ALL ATTRIBUTES!!! IT WILL SOMEDAY!!!!
#######################################################################
sub GetParaAttributeList {
    local(*para, *Attr, *tabstops) = @_ ;
    local(@ltabstops)              = () ;

    $Attr{"PgfTag"} = $v
         if ($v = &get_str(&first_match(*para, "<PgfTag\\s(.+)")));

    $Attr{"PgfFIndent"} = $v
         if ($v = &get_dec(&first_match(*para, "<PgfFIndent\\s(.+)")));

    $Attr{"PgfLIndent"} = $v
         if ($v = &get_dec(&first_match(*para, "<PgfLIndent\\s(.+)")));

    $Attr{"PgfRIndent"} = $v
         if ($v = &get_dec(&first_match(*para, "<PgfRIndent\\s(.+)")));

    $Attr{"PgfAutoNum"} = $v
         if ($v = &get_YN(&first_match(*para, "<PgfAutoNum\\s(.+)")));

    $Attr{"PgfNumFormat"} = $v
         if ($v = &get_str(&first_match(*para, "<PgfNumString\\s(.+)")));

    $Attr{"PgfNumString"} = $v
         if ($v = &get_str(&first_match(*para, "<PgfNumString\\s(.+)")));

    $Attr{"PgfNumberFont"} = $v
         if ($v = &get_str(&first_match(*para, "<PgfNumberFont\\s(.+)")));

    $Attr{"PgfNumAtEnd"} = $v
         if ($v = &get_YN(&first_match(*para, "<PgfNumAtEnd\\s(.+)")));

    $Attr{"PgfNumTabs"} = $v
         if ($v = &get_num(&first_match(*para, "<PgfNumTabs\\s(.+)")));

    $Attr{"PgfFont"} = $v
         if ($v = &get_str(&first_match(*para, "<PgfFont\\s([^>]+>)")));

    $Attr{"FFamily"} = $v
         if ($v = &get_str(&first_match(*para, "<FFamily\\s([^>]+>)")));

    $Attr{"PgfPlacementStyle"} = $v
         if ($v = &get_item(&first_match(*para, "<PgfPlacementStyle\\s(.+)")));

    $Attr{"PgfPlacement"} = $v
         if ($v = &get_item(&first_match(*para, "<PgfPlacement\\s(.+)")));

    $Attr{"PgfAlignment"} = $v
         if ($v = &get_item(&first_match(*para, "<PgfAlignment\\s(.+)")));

    foreach (&all_matches(*para, "<TabStop")) {
        $tabpos = &get_dec($1) if (/<TSX\s+([^>]+>)/) ;
        $tabalign = $1 if (/<TSType\s+([^>\s]+)\s*>/) ;
        $tablead  = $1 if (/<TSLeaderStr\s+([^>\s]+)\s*>/) ;
        push(@ltabstops, "$tabpos:$tabalign:$tablead") ;
    }
    @tabstops = @ltabstops if (@ltabstops);
}


sub get_str {
    return $1 if ($_[0] =~ /^\s*`([^\']+)'/);
    return "" ;
}

sub get_dec {
    local($foo) = @_ ;
    return &get_inches(*foo) if ($foo);
    return "" ;
}

sub get_inches {
    local(*string) = @_ ;

    if ($string =~ s/^\s*(\d*\.*\d*)\s*([^>\s]*)//) {
        return &convert_to_inches($1, $2);
    }
    return "" ;
}

sub get_num {
    return $1 if ($_[0] =~ /^\s*(\d+)/);
    return "" ;
}

sub get_dim {
    local($S, $i)  = @_ ;
    local($dim)    = "" ;

    return "" if (!$_[0]);
    $dim = join('$;', $dim, &get_inches(*S)) while ($i--) ;   
    return $dim ;
}

sub get_item {
    return $1 if ($_[0] =~ /^\s*(\w+)/);
    return "" ;
}

sub get_YN {
    return ($_[0] =~ /Yes/);
}

# don't know if this ever will or should get used
sub get_tokenname {
    return $1 if ($_[0] =~ /^\s*<(\w+)/);
    return "" ;
}

sub convert_to_inches {
    local($size, $units) = @_ ;
    local($global_units) = &get_global_units;

    $units = $global_units if (!$units);   # if not overridden use default

    &Out(4, "Converting $size $units to inches");
    if (($units !~ /"/) && ($units !~ /in/)) {
        return ($_[0]) if ($units eq "");
        return ($size / 72) if ($units =~ /pt/);
        return ($size / 6) if ($units =~ /pica/);
        return ($size / 2.54) if ($units =~ /cm/);
        return ($size / 254) if ($units =~ /mm/);
        return (($size * .375) / 254) if ($units =~ /dd/);
        return (($size * 4.55) / 254) if ($units =~ /cc/);
    }
    &Out(4, " er... already in inches :)", 1);
    return $size ;
}


sub get_global_units {
    local($UNITS) = "" ; # if none specified, default will be Uin (done below)

    $UNITS = &first_match(*TOP, "\\s*<Units\\s+U([^\\s]+)");

    return ($UNITS) ? $UNITS : "Uin" ;
}

1;
