###############################################################################
# parse.pl -- sucks up named mif file into memory 
###############################################################################


########################################################################
# get_chunk -- gets the next <token> ignoring nested tokens
########################################################################
sub get_chunk {
    package MIFSTRUCT ;
    local (*letters) = @_ ;
    local ($brackets) = 0 ;
    local ($chunk) = "" ;
    local ($text) = "";
    local ($image) = "" ;
    local ($token) ;
    local ($l, $i, $j);

#------------------------------------------------------------------------
# suck up the beginning of the token
#-----------------------------------------------------------------------
    while ((@letters != 0) && (!($chunk =~ /</))) {   # suck up to first token
       $chunk .= shift @letters ;
#      return ("","") if (@letters == 0) ;
    }
    $brackets = 1 ;

    if (@letters == 0) {
        return ("","") ;
    }
#-----------------------------------------------------------------------
# suck up the rest of the token
#-----------------------------------------------------------------------
    do {
        if (!@letters) {
           print "\nError: Parser unable to find token inside of $miffile!";
           return ("","");
        }
        $l = shift @letters ;
        $brackets-- if ($l eq ">") ;
        $brackets++ if ($l eq "<") ;
        if ($l eq "`") {
            $l .= shift @letters until ($l =~ /'$/) ;
        }
        if ($l eq "#") {
            $l .= shift @letters until ($l =~ /\n$/) ;
        }
        if ($l eq "=") {
            $j = 0 ;
            $image = "" ;
            do { $image .= $letters[$j++] ; } while ($image !~ /\n$/) ;
            chop $image ;
            
            if ($image =~ /^EndInset$/) {
#                print "\nOkay, we've found the end of the image!" ;
                do { $l .= shift @letters ; } while ($l !~ /\n$/) ;
            }
            elsif ($image =~ /^FrameImage$/) {
#                print "\nOkay, we've found a FrameImage!" ;
                $l .= shift @letters until ($letters[0] eq "&") ;
                while ($letters[0] eq "&") {
                   $l .= shift @letters until ($letters[0] eq "\n") ;
                   $l .= shift @letters ;
                }
#                print "\nDone with the FrameImage!" ;
            }
            elsif ($image =~ /^EPSI$/) {
#                print "\nOkay, we've found an EPSI!" ;
                $l .= shift @letters until ($letters[0] eq "&") ;
                while ($letters[0] eq "&") {
                   $l .= shift @letters until ($letters[0] eq "\n") ;
                   $l .= shift @letters ;
                }
#                print "\nDone with the EPSI!" ;
            }
            else {
                print "\nError:  Unexpected ImportObject type: $image!";
                return ("", "");
            }
        }
        $chunk .= $l ;
    } while ($brackets > 0) ;

#-------------------------------------------------------------------------
# suck up the rest of the line --  This shouldn't suck up the rest of the 
#                                  line, it should really suck up until the 
#                                  end of the line or until the next token.  
#                                  This is a hack dependent upon Frame-
#                                  generated mif, which needs to be fixed.
#--------------------------------------------------------------------------
    while ((@letters != 0) && (!($text =~ /\n/))) {
       $text .= shift @letters ;
    }
    $chunk .= $text ;

#-------------------------------------------------------------------------
# identify the token
#-------------------------------------------------------------------------
    if ($chunk =~ /^[ \t]*<([A-Za-z]+)/) {
       $token = $1 ;
       return ($token, $chunk);
    }

    return ("","");
}




#########################
# get_MIFFile -- suck up MIFFile token and Frame version
#########################
sub get_MIFFile {
    package MIFSTRUCT ;

    &Out(2, "Getting MIFFile...");
    $MIFFile = shift @_ ;
    $Version = $1 if ($MIFFile =~ /MIFFile (\d.\d)/) ;
}




#########################
# get_Units -- suck up the default Units section
#########################
sub get_Units {
    package MIFSTRUCT ;

    &Out(2, "Getting Units...");
    $Units = shift @_ ;
}

#########################
# get_Verbose -- suck up the default Verbose setting
#########################
sub get_Verbose {
    package MIFSTRUCT ;

    &Out(2, "Getting Verbose...");
    $Verbose = shift @_ ;
}



#########################
# ColorCatalog
#########################
sub get_ColorCatalog {
    package MIFSTRUCT ;

    &Out(2, "Getting Color Catalog...");
    $ColorCatalog = shift @_ ;
}


#########################
# Condition Formats
#########################
sub get_ConditionCatalog {
    package MIFSTRUCT ;

    &Out(2, "Getting Condition Catalog...");
    $ConditionCatalog = shift @_ ;
}



#########################
# Paragraph Catalog
#########################
sub get_PgfCatalog {
    package MIFSTRUCT ;
    local(@loc) ;
    local($newpgf) ;

    &Out(2, "Getting Paragraph Catalog...");
    if (@_[0] =~ s/^[ \t\n]*<PgfCatalog[ \t]*\n//) {
        @loc = split(//,@_[0]);
        ($token, $chunk) = &get_chunk(*loc);
        while ($token ne "") {
           if ($PgfCatalog_tokens{$token} eq "") {
               &Err("Illegal token in PgfCatalog: $token");
               &Err("Skipping rest of Catalog.");
               return 0 ;
           }
           if ($token eq "Pgf") {
               $newpgf = &get_Pgf($chunk) ;
               push (@PgfCatalog, $newpgf) if ($newpgf);
           }
           else {
               push (@PgfCatalog, $chunk) ;
           }
           ($token, $chunk) = &get_chunk(*loc);
        }
        return 1 ;
    }
    &Err("PgfCatalog incorrectly defined.  Skipping...");
    return 0 ;
}

sub get_Pgf {
    package MIFSTRUCT ;
    local (@loc);
    local (@newpgf) ;
    local ($newname);

    &Out(3, "Getting Paragraph Format...");
    if (@_[0] =~ s/^[ \t\n]*<Pgf[ \t]*\n//) {
        @loc = split(//, @_[0]);
        ($token, $chunk) = &get_chunk(*loc);
        while ($token ne "") {
           if ($Pgf_tokens{$token} eq "") {
             &Err("Illegal token in Pgf: $token");
             &Err("Skipping rest of Pgf.");
             return 0 ;
           }
           push (@newpgf, $chunk);
           ($token, $chunk) = &get_chunk(*loc);
        }
        $newname = "Pgf" . ($#PgfCatalog+1) ;
        eval '@' . $newname . '= @newpgf' ;
        return $newname ;
    }
    &Err("Pgf incorrectly defined.  Skipping...");
    return 0 ;
}



#########################
# Element Definition Catalog
#########################
sub get_ElementDefCatalog {
    package MIFSTRUCT ;

    &Out(2, "Getting Element Definition Catalog...");
    $ElementDefCatalog = shift @_ ;
}


#########################
# Font Catalog
#########################
sub get_FontCatalog {
    package MIFSTRUCT ;

    &Out(2, "Getting Font Catalog...");
    $FontCatalog = shift @_ ;
}

#########################
# Ruling Catalog
#########################
sub get_RulingCatalog {
    package MIFSTRUCT ;

    &Out(2, "Getting Ruling Catalog...");
    $RulingCatalog = shift @_ ;
}


#########################
# Table Formats
#########################
sub get_TblCatalog {
    package MIFSTRUCT ;

    &Out(2, "Getting Table Catalog...");
    $TblCatalog = shift @_ ;
}


#########################
# Views 
#########################
sub get_Views {
    package MIFSTRUCT ;

    &Out(2, "Getting Views...");
    $Views = shift @_ ;
}



#########################
# VariableFormats Formats
#########################
sub get_VariableFormats {
    package MIFSTRUCT ;

    &Out(2, "Getting Variable Formats...");
    $VariableFormats = shift @_ ;
}


#########################
# XRefFormats
#########################
sub get_XRefFormats {
    package MIFSTRUCT ;

    &Out(2, "Getting XRef Formats...");
    $XRefFormats = shift @_ ;
}


#########################
# Document Formats
#########################
sub get_Document {
    package MIFSTRUCT ;

    &Out(2, "Getting Document Properties...");
    $Document = shift @_ ;
}


#########################
# Book Components
#########################
sub get_BookComponent {
    package MIFSTRUCT ;

    &Out(2, "Getting a Book Component...");
    push(@BookComponents, shift @_) ;
}


#########################
# Initial Auto Nums
#########################
sub get_InitialAutoNums {
    package MIFSTRUCT ;

    &Out(2, "Getting Initial Autonums...");
    $InitialAutoNums = shift @_ ;
}



#########################
# Dictionary
#########################
sub get_Dictionary {
    package MIFSTRUCT ;

    &Out(2, "Getting Dictionary...");
    $Dictionary = shift @_ ;
}


#########################
# AFrames
#########################
sub get_AFrames {
    package MIFSTRUCT ;
    local(@loc) ;
    local($newframe);

    &Out(2, "Getting AFrames...");
    if (@_[0] =~ s/^[ \t\n]*<AFrames[ \t]*\n//) {
       @loc = split(//, @_[0]);
       ($token, $chunk) = &get_chunk(*loc) ;
       while ($token ne "") { 
          if ($token ne "Frame") {
             &Err("Illegal token in Anchored Frames: $token  Skipping...");
             return 0 ;
          }
          $newframe = &get_Frame($chunk) ;
          push (@AFrames, $newframe) if ($newframe);
          ($token, $chunk) = &get_chunk(*loc) ;
       }
       return 1 ;
    }
    &Err("AFrames incorrectly defined.  Skipping...");
    return 0 ;
}


sub get_Frame {
    package MIFSTRUCT ;
    local (@loc) ;
    local (@newframe) ;
    local ($newname) ;

    &Out(3, "Getting a frame...");
    if (@_[0] =~ s/^[ \t\n]*<Frame[ \t]*\n//) {
        @loc = split(//, @_[0]);
        ($token, $chunk) = &get_chunk(*loc);
        while ($token ne "") {
           if ($Frame_tokens{$token} eq "") {
             &Err("Illegal token in Frame: $token  Skipping...");
             return 0 ;
           }
           push (@newframe, $chunk);
           ($token, $chunk) = &get_chunk(*loc);
        }
        $newname = "Frame" . ($#AFrames+1) ;
        eval '@' . $newname . '= @newframe' ;
        return $newname ;
    }
    &Err("Frame incorrectly defined.  Skipping...");
    return 0 ;
}
    


#########################
# Tbls
#########################
sub get_Tbls {
    package MIFSTRUCT ;
    local(@loc) ;
    local($newtable);

    &Out(2, "Getting Tables...");
    if (@_[0] =~ s/^[ \t\n]*<Tbls[ \t]*\n//) {
       @loc = split(//, @_[0]);
       ($token, $chunk) = &get_chunk(*loc) ;
       while ($token ne "") {
          if ($token ne "Tbl") {
             &Err("Illegal token in Tables: $token  Skipping...");
             return 0 ;
          }
          $newtable = &get_Tbl($chunk) ;
          push (@Tbls, $newtable) if ($newtable);
          ($token, $chunk) = &get_chunk(*loc) ;
       }
       return 1 ;
    }
    &Err("Tbls incorrectly defined.  Skipping...");
    return 0 ;
}


sub get_Tbl {
    package MIFSTRUCT ;
    local (@loc) ;
    local (@newtable) ;
    local ($newname) ;

    &Out(3, "Getting a table...");
    if (@_[0] =~ s/^[ \t\n]*<Tbl[ \t]*\n//) {
        @loc = split(//, @_[0]);
        ($token, $chunk) = &get_chunk(*loc);
        while ($token ne "") {
           if ($Tbl_tokens{$token} eq "") {
             &Err("Illegal token in Tbl: $token  Skipping...");
             return 0 ;
           }
           push (@newtable, $chunk);
           ($token, $chunk) = &get_chunk(*loc);
        }
        $newname = "Tbl" . ($#Tbls+1) ;
        eval '@' . $newname . '= @newtable' ;
        return $newname ;
    }
    &Err("Frame incorrectly defined.  Skipping...");
    return 0 ;
}


#########################
# Page
#########################
sub get_Page {
    package MIFSTRUCT ;
    local(@loc);
    local(@newpage);          # local storage of page we are building
    local($newname);          # name of global storage of page we are building

    &Out(2, "Getting a page...");
    if (@_[0] =~ s/^[ \t\n]*<Page[ \t]*\n//) {
       @loc = split(//, @_[0]);
       ($token, $chunk) = &get_chunk(*loc) ;
       while ($token ne "") {
          if ($Page_tokens{$token} eq "") {
             &Err("Illegal token in Page: $token  Skipping...");
             return 0 ;
          }
          push (@newpage, $chunk) ;
          ($token, $chunk) = &get_chunk(*loc) ;
       }

       $newname = &get_new_page_name(*newpage);
       push(@Pages, $newname);
       eval '@' . $newname . '= @newpage' ;
       return 1 ;
    }
    &Err("Page incorrectly defined.  Skipping...");
    return 0 ;
}


sub get_new_page_name {
    package MIFSTRUCT ;
    local(*page) = @_ ;
    local(*name) ;

    $name  = &first_match(*page,"<PageType\\s+(\\w+)Page\\s*>");
    $name .= &first_match(*page, "<PageTag\\s+`(\\w+)'\\s*>");
    $name .= &first_match(*page, "<PageNum\\s+`(\\d+)'\\s*>");

    return $name ;
}




#########################
# TextFlow
#########################
sub get_TextFlow {
    package MIFSTRUCT ;
    local(@loc);
    local(@newflow);          # local storage of flow we are building
    local($newname);          # name of global storage of flow we are building

    &Out(2, "Getting a textflow...");
    if (@_[0] =~ s/^[ \t\n]*<TextFlow[ \t]*\n//) {
       @loc = split(//, @_[0]);
       ($token, $chunk) = &get_chunk(*loc) ;
       while ($token ne "") {
          if ($TextFlow_tokens{$token} eq "") {
             &Err("Illegal token in TextFlow: $token  Skipping...");
             return 0 ;
          }
          if ($token eq "Para") {
              push(@newflow, &get_Para($chunk));
          }
          else {
              push (@newflow, $chunk) ;
          }
          ($token, $chunk) = &get_chunk(*loc) ;
       }

       $newname  = "TextFlow" . ($#TextFlows+1) ;

       push(@TextFlows, $newname);
       eval '@' . $newname . '= @newflow' ;
       return 1 ;
    }
    &Err("TextFlow incorrectly defined.  Skipping...");
    return 0 ;
}


sub get_Para {
    package MIFSTRUCT ;
    local(@loc);
    local(@newpara);
    local($newname);

    &Out(3, "Getting a paragraph...");
    if (@_[0] =~ s/^[ \t\n]*<Para[ \t]*\n//) {
       @loc = split(//, @_[0]);
       ($token, $chunk) = &get_chunk(*loc) ;
       while ($token ne "") {
          if ($Para_tokens{$token} eq "") {
             &Err("Illegal token in Para: $token  Skipping...");
             return 0 ;
          }
          if ($token eq "ParaLine") {
              push(@newpara, &get_ParaLine($chunk));
          }
          else {
              push (@newpara, $chunk) ;
          }
          ($token, $chunk) = &get_chunk(*loc) ;
       }

       $newname  = "Para" . ($#Paras+1) ;

       push(@Paras, $newname);
       eval '@' . $newname . '= @newpara' ;
       return $newname ;
    }
    &Err("Para incorrectly defined.  Skipping...");
    return 0 ;
}


sub get_ParaLine {
    package MIFSTRUCT ;
    local(@loc);
    local(@newparaline);
    local($newname);

    &Out(4, "Getting a ParaLine...");
    if (@_[0] =~ s/^[ \t\n]*<ParaLine[ \t]*\n//) {
       @loc = split(//, @_[0]);
       ($token, $chunk) = &get_chunk(*loc) ;
       while ($token ne "") {
          if ($ParaLine_tokens{$token} eq "") {
             &Err("Illegal token in ParaLine: $token  Skipping...");
             return 0 ;
          }
          push (@newparaline, $chunk) ;
          ($token, $chunk) = &get_chunk(*loc) ;
       }

       $newname  = "ParaLine" . ($#ParaLines+1) ;

       push(@ParaLines, $newname);
       eval '@' . $newname . '= @newparaline' ;
       return $newname ;
    }
    &Err("ParaLine incorrectly defined.  Skipping...");
    return 0 ;
}


sub dechunk {
    package MIFSTRUCT ;
    local($tokenname) = @_[0] ;
    local(@newarray);
    local(%legal_tokens);
    local(@loc);

    return "" if (!$tokenname) ;      # Gotta tell us what token we're parsing!!

    &Out(2, "Getting a $tokenname...");
    if (@_[1] =~ s/^[ \t\n]*<${tokenname}[ \t]*\n//) {
       @loc = split(//, @_[1]);
       ($token, $chunk) = &get_chunk(*loc) ;
       while ($token ne "") {
          eval '%legal_tokens=%' . $tokenname . '_tokens' ;
          if ($legal_tokens{$token} eq "") {
             &Err("Illegal token in $tokenname: $token.  Skipping...");
             return "" ;
          }
          push (@newarray, $chunk) ;
          ($token, $chunk) = &get_chunk(*loc) ;
       }
       return @newarray ;
    }
    &Err("$tokenname incorrectly defined.  Skipping...");
    return "" ;
}

1;
