###############################################################################
# tables.pl -- interface to Tables
###############################################################################

require "tables.map" ;

########################################
# get_table_from_id 
########################################
sub get_table_from_id {
    package MIFSTRUCT ;
    local($ID) = @_ ;
    local(@table);
    local($pattern);
    local($tablename) = "";

    foreach (@Tbls) {
       @table =  &get_array($_) ;
       $pattern = "<TblID\\s+($ID)>" ;
       $tablename = &first_match(*table, $pattern);
       if ($tablename) {
          return $_ ;
       }
    }
    return "" ;
}



###########################################################
# get_table_title(*table) -- returns the title (if any) of
#            a table.  Requires the TextFlow ID it exists in
#            Specifying TRUE for html saves markers, fonts
###########################################################
sub get_table_title {
    package MIFSTRUCT ;
    local($tablename, $ID, $sgml) = @_ ;
    local(@table);
    local($text)     = "" ;
    local($paraname) = "";
    local(@para);
    local($line);
    local($cont);

    @table = &get_array($tablename);
    foreach (@table) {
       if (/<TblTitleContent/) {
          foreach (&dechunk("TblTitleContent", $_)) {
              if (/^\s*<Para[\s]/) {
                 $paraname = &get_Para($_) ;
                 if ($paraname) {
                     @para = &get_array($paraname) ;
                     $line = "" ;
                     if ($sgml) {
                         ($line, $cont) = &htmlize_paras(*para, $ID, 1, 1) ;
                         $text .= $line ;
                     }
                     else {
                         ($line, $cont) = &get_text_in_para(*para, $ID);
                         $text .= $line ;
                     }
                 }
              }
          }
          last ;
       }
    }
    return $text ;
}


sub get_table_text {
    package MIFSTRUCT ;
    local($tablename, $ID, $TABLE_PADDING) = @_ ;
    local(@table)          = () ;
    local($numcolumns)     = 0  ;
    local($text)           = "" ;
    local($i)              = 0  ;
    local($j)              = 0  ;
    local($k)              = 0  ;
    local(@items)          = () ;
    local(@rows)           = () ;
    local(@cells)          = () ;
    local($pname)          = "" ;
    local($ptext)          = "" ;
    local($row)            = 0  ;
    local($col)            = 0  ;
    local($headrows)       = 0  ;
    local($bodyrows)       = 0  ;
    local($footrows)       = 0  ;
    local($numrows)        = 0  ;
    local($longest_line)   = 0  ;
    local($cont)           = "" ;
    local(@para)           = () ;
    local($len)            = 0  ;
    local($buf)            = "" ;
    local($line)           = "" ;
    local(@numlines)       = () ;
    local(@maxlines)       = () ;
    local(@longest)        = () ;
    local(@num_lines)      = () ;
    local($tablewidth)     = 0  ;
    local($columnwidth)    = 0  ;
    local($TblIndent)      = "" ; 
    local($cellwrapwidth)  = 0  ;
    local($HeadWidth)      = 0  ;
    local($BodyWidth)      = 0  ;
    local($FootWidth)      = 0  ;
    local($tmpstr)         = "" ;

    $cellwrapwidth = length($TableWrappers{"OpenCell"})  + 
                     length($TableWrappers{"CloseCell"}) ;

    @table = &get_array($tablename) ;
    foreach (@table) {
       if (/<TblNumColumns\s+(\d+)\s*>/) {
           $numcolumns = $1 ;
           next ;
       }
       if (/<TblH[\s]/) {
           $headrows = 0 ;
           @items = &dechunk("TblH", $_) ;
           foreach (@items) {
              if (/<Row[\s]/) {
                  @rows = &dechunk("Row", $_) ;
                  $row++ ;
                  $headrows++ ;
                  $col = 0 ;
                  foreach (@rows) {
                     if (/<Cell[\s]/) {
                         $col++ ;
                         @cells = &dechunk("Cell", $_) ;
                         foreach (@cells) {
                            if (/<CellContent[\s]/) {
                                @content = &dechunk("CellContent", $_) ;
                                foreach (@content) {
                                   next if (!(/^\s*<Para[\s]/));
                                   if (/^\s*<Para[\s]/) {
                                      $pname = &get_Para($_) ;
                                      if ($pname) {
                                         @para = &get_array($pname) ;
             ($ptext, $cont) = &get_text_in_para(*para,$ID);
             $ptext = " " if ($ptext eq "") ;
             @entries = split('\n', $ptext);
             push(@entries, " ");
             foreach (@entries) { 
                s/$/ / ; 
             }
             eval 'push(@cell' . $row . 'x' . $col . ',@entries)' ;
                                      } # if
                                   } # if
                                } # for
                            } # if
                         } # for
                     } # if
                  } # for
              } # if
           } # for
       } # if
       if (/<TblBody[\s]/) {
           $bodyrows = 0 ;
           @items = &dechunk("TblBody", $_) ;
           foreach (@items) {
              if (/<Row[\s]/) {
                  @rows = &dechunk("Row", $_) ;
                  $row++ ;
                  $bodyrows++ ;
                  $col = 0 ;
                  foreach (@rows) {
                     if (/<Cell[\s]/) {
                         $col++ ;
                         @cell = &dechunk("Cell", $_) ;
                         foreach (@cell) {
                            if (/<CellContent[\s]/) {
                                @content = &dechunk("CellContent", $_) ;
                                foreach (@content) {
                                   if (/^\s*<Para[\s]/) {
                                      $pname = &get_Para($_) ;
                                      if ($pname) {
                                         @para = &get_array($pname) ;
             ($ptext, $cont) = &get_text_in_para(*para,$ID);
             $ptext = " " if ($ptext eq "\n") ;
             @entries = split('\n', $ptext);
             push(@entries, " ");
             foreach (@entries) { 
                $_ .= ' '  ;
             }
             eval 'push(@cell' . $row . 'x' . $col . ',@entries)' ;
                                      }
                                   }
                                }
                            }
                         }
                     }
                  }
              }
           }
       }
       if (/<TblF[\s]/) {
           $footrows = 0 ;
           @items = &dechunk("TblF", $_) ;
           foreach (@items) {
              if (/<Row[\s]/) {
                  @rows = &dechunk("Row", $_) ;
                  $row++ ;
                  $footrows++ ;
                  $col = 0 ;
                  foreach (@rows) {
                     if (/<Cell[\s]/) {
                         $col++ ;
                         @cells = &dechunk("Cell", $_) ;
                         foreach (@cells) {
                            if (/<CellContent[\s]/) {
                                @content = &dechunk("CellContent", $_) ;
                                foreach (@content) {
                                   if (/^\s*<Para[\s]/) {
                                      $pname = &get_Para($_) ;
                                      if ($pname) {
                                         @para = &get_array($pname) ;
             ($ptext, $cont) = &get_text_in_para(*para,$ID);
             $ptext = " " if ($ptext eq "") ;
             @entries = split('\n', $ptext);
             push(@entries, " ");
             foreach (@entries) { 
                s/$/ / ; 
             }
             eval 'push(@cell' . $row . 'x' . $col . ',@entries)' ;
                                      } # if
                                   } # if
                                } # for
                            } # if
                         } # for
                     } # if
                  } # for
              } # if
           } # for
       } # if
    }


    if ($col != $numcolumns) {
        if ($col < $numcolumns) {
            &Err("Table $table doesn't have all columns defined!");
        }
        elsif ($col > $numcolumns) {
            &Err("Table $table had more columns defined than needed!");
        }
        &Err("Winging it, using $col columns rather than $numcolumns");
    }
    
    &Out(2, "Table: $tablename has $row rows and $col columns");
    for ($i=1; $i<=$col; $i++) {
         $longest[$i] = 1 ;
         for ($j=1; $j <= $row; $j++) {
              eval 'pop(@cell' . $j . 'x' . $i . ')' ;
              eval '@entries=@cell' . $j . 'x' . $i ;
              $longest_line = 0 ;
              foreach (@entries) {
                   $tmpstr = $_ ;
                   $tmpstr =~ s/&\w+;/#/g;   # HACK  for HTML and SGML entites
                                             # should be a callback $callback
                   $len = length($tmpstr) ;
                   $longest_line = $len if ($len > $longest_line);
              }
              $longest[$i] = $longest_line if ($longest_line > $longest[$i]);
         }
    }
         

    for ($i=1; $i <= $row; $i++) {
         @numlines = () ;
         $numlines[0] = 1 ;
         for ($j=1; $j <= $col; $j++) {
             eval '$numlines[$j]=@cell' . $i . 'x' . $j ;
         }
         @num_lines = sort bynumber @numlines ;
         $maxlines[$i] = $num_lines[$#num_lines] ; 
    }
      
    $HeadWidth = &get_table_width("Head");
    $BodyWidth = &get_table_width("Body");
    $FootWidth = &get_table_width("Foot");
    $tablewidth = &max3($HeadWidth, $BodyWidth, $FootWidth);
    
    if ($centertable) {
        $TblIndent = ' ' x (int((80 - $tablewidth) / 2)) if ($tablewidth < 80); 
    }

#---------------------------------------------------------------------------
#  AT THIS POINT:
#
#  maxlines[]  has number of lines for each row
#  longest[]   has longest string for each column
#  cellNxM     has strings for cell M of row N (there are maxlines[N] strings)
#---------------------------------------------------------------------------

    $text .= $TableWrappers{"OpenTable"} ;
    $text .= $TableWrappers{"OpenTableGroup"} ;
    $text .= &create_table_rule("Top");
    if ($headrows) {
        $text .= $TableWrappers{"OpenHead"} ;
        $text .= &create_table_row ("Head", 1, $headrows) ;
        $text .= &create_table_rule("HB") if ($headrows && $bodyrows) ;
        $text .= $TableWrappers{"CloseHead"} ;
    }
    if ($bodyrows) {
        $text .= $TableWrappers{"OpenBody"} ;
        $text .= &create_table_row ("Body", $headrows+1, $headrows+$bodyrows);
        $text .= &create_table_rule("BF") if ($bodyrows && $footrows) ;
        $text .= $TableWrappers{"CloseBody"} ;
    }
    if ($footrows) {
        $text .= $TableWrappers{"OpenFoot"} ;
        $text .= &create_table_row ("Foot", $headrows+$bodyrows+1, 
                                            $headrows+$bodyrows+$footrows);
        $text .= $TableWrappers{"CloseFoot"} ;
    }
    $text .= &create_table_rule("Bottom");
    $text .= $TableWrappers{"CloseTableGroup"} ;
    $text .= $TableWrappers{"CloseTable"} ;
   

#--------------------------------------------------------------------------
# Clean up time
#--------------------------------------------------------------------------
    for ($i=1; $i <= $row; $i++) {
         for ($j=1; $j <= $col; $j++) {
             eval 'undef @cell' . $i . 'x' . $j ;
         }
    }

    return $text ;
}


sub create_table_row {
    local($type, $first, $last) = @_ ;
    local($line)  = "" ; # TblIndent centers table
    local($i)     = 0  ;
    local($j)     = 0  ;
    local($k)     = 0  ;
    local($LEFT)  = "" ;
    local($COLUMN)= "" ;
    local($RIGHT) = "" ;
    local($pos)   = "" ;

    return "" if ($last < $first);
    return "" if (($last == 0) || ($last eq "0"));

    $pos    = $type . "Left" ;
    $LEFT   = $TblChars{$pos} || "" ;
    $pos    = $type . "Col" ;
    $COLUMN = $TblChars{$pos} || "" ;
    $pos    = $type . "Right" ;
    $RIGHT  = $TblChars{$pos} || "" ;

    #-------------------------------------------------------------------
    # Construct First through Last rows (with ruling between inner rows)
    #
    #    $LEFT texttext $COLUMN texttext $COLUMN texttext $RIGHT
    #    -------------------------------------------------------
    #    $LEFT texttext $COLUMN texttext $COLUMN texttext $RIGHT
    #    -------------------------------------------------------
    #    $LEFT texttext $COLUMN texttext $COLUMN texttext $RIGHT
    #
    #-------------------------------------------------------------------
    for ($i=$first; $i <= $last ; $i++) {
         $line .= $TblIndent ;
         $line .= $TableWrappers{"OpenRow"};
         for ($j=0; $j< $maxlines[$i]; $j++) {
              $line .= $LEFT ;

              for ($k=1; $k <= $col; $k++) {
                   eval '$_ = $cell' . $i . 'x' . $k . '[$j] || " "' ;
                   $line .= join("", 
                                     $TableWrappers{"OpenCell"},
                                     $_,
                                     $TableWrappers{"CloseCell"}
                            );
                   if ($TABLE_PADDING) {
                       if ($longest[$k] > length($_)) {
                           $line .= ' ' x ($longest[$k] - length($_)) ;
                       }
                   }
                   $line .= $COLUMN  if ($k != $col) ;
              }

              $line .= "$RIGHT\n" ;
         }
         $line .= &create_table_rule($type) if ($i != $last);
         $line .= $TableWrappers{"CloseRow"};
    }
    return $line ;
}


##############################################################################
# construct_table_rule --
#   builds a single rule of the table, terminated with a newline.  The type of
#   row is passed in and can be one of "Top", "Head", "Body", or "Foot".  The
#   routine uses various variables local to its caller, so this routine is
#   not meant to be called by anyone but get_table_text.
#
#   You have been warned.
#
##############################################################################
sub create_table_rule {
    package MIFSTRUCT ;
    local($type)        = @_ ;
    local($line)        = $TblIndent ;  # TblIndent centers table
    local($i)           = 0  ;
    local($LEFT)        = "" ;
    local($MAIN)        = "" ;
    local($COLUMN)      = "" ;
    local($RIGHT)       = "" ;
    local($pos)         = "" ;

    $pos = $type . "Rule" ;
    $MAIN   = $TblChars{$pos} || "" ;
    $pos = $type . "RuleLeft" ;
    $LEFT   = $TblChars{$pos} || "" ;
    $pos = $type . "RuleCol" ;
    $COLUMN = $TblChars{$pos} || "" ;
    $pos = $type . "RuleRight" ;
    $RIGHT  = $TblChars{$pos} || "" ;
    
    $line .= $LEFT ;
    for ($i=1; $i <= $col; $i++) {
         $line .= ($MAIN x ($longest[$i]+$cellwrapwidth)) ;
         $line .= $COLUMN if ($i != $col);
    }
    $line .= "$RIGHT\n" ;
    return $line ;
}


    
sub get_table_width {
    package MIFSTRUCT ;
    local($type)        = @_ ;
    local($i)     = 0 ;
    local($width) = 0 ;
    local($pos)   = "" ;

    $pos = $type . "Rule" ;
    $MAIN   = $TblChars{$pos} || "" ;
    $pos = $type . "RuleLeft" ;
    $LEFT   = $TblChars{$pos} || "" ;
    $pos = $type . "RuleCol" ;
    $COLUMN = $TblChars{$pos} || "" ;
    $pos = $type . "RuleRight" ;
    $RIGHT  = $TblChars{$pos} || "" ;
    
    $width += length($LEFT) ;
    for ($i=1; $i <= $col; $i++) {
        $width += (length($MAIN)*($longest[$i] + $cellwrapwidth)) ;
        $width += length($COLUMN) ;
    }
    $width += length($RIGHT) ;

    return $width ;
}

1;
