#############################################################################
# util.pl -- Utility functions used by perl scripts
#############################################################################

sub get_abbr_args {
    local(*argdefs, $help_routine, @args) = @_ ;
    local($i)              = 0 ;
    local(@ARGV)           = ();

    for ($i=0; $i < @args; $i++) {
        if (!($args[$i] =~ /^-/)) {
            push(@ARGV, $args[$i]);
            next ;
        }
        $args[$i] =~ s/^-// ;
        $found = 0 ;
        foreach (keys(%argdefs)) {
           if (/^$args[$i]/) {
              $found = 1 ;
              if ($argdefs{$_} eq "0") {
                  eval '$' . $_ . '= 1' ;
                  $found = 1 ;
                  last ;
              }
              if ($i == $#args) {
                  print "\nError: flag -$_ requires an argument. Exiting...\n";
                  exit 1 ;
              }
              if ($argdefs{$_} eq "a") {
                  eval 'push(@' . $_ . ',$args[++$i])' ;
                  last ;
              }
              eval '$' . $_ . '="' . $args[++$i] . '"' ;
              last ;
           }
        }
        if (!$found) {
           &$help_routine($args[$i]) if ($help_routine) ;
           print "\nError: Unknown flag -$args[$i].  Exiting...\n";
           exit 1 ;
        }
    }
    return @ARGV ;
}



########################################################################
# Prompt -- displays a prompt (useful for debugging, pass prompt string)
########################################################################
sub prompt {
    print STDOUT "\n<$_[0]> " ;
    while (<STDIN>) {
       return if (/exit/);
       eval $_ ;
       warn $@ if $@ ;
       print STDOUT "\n<$_[0]> " ;
    }
}


sub MOut {
    local($level, $message, $sameline) = @_ ;
    local($length)                     = 0  ;  
    local($prefix)                     = "" ;
    local($postfix)                    = "" ;
    local($filter)                     = "" ;

    #-------------------------------------------------------------------
    # This routine uses two "global" variables:
    #   $'DD       debugging detail level (only set in mmf territory)
    #   $'FILTER   name of current filter (might not be set in all cases)
    #
    # If DD isn't defined, then this routine is just being used for
    # general output (not in mmf sense) and so it simply outputs message
    #
    # If FILTER isn't defined, then this routine is just being used for
    # general controlled output, and so it simply outputs if DD is high
    # enough (DD > level).  
    #-------------------------------------------------------------------

    if (defined($'DD))  {
        if ($'DD > $level) {
            if (defined($sameline) && $sameline) {
                print STDOUT $message ;
                return ;
            }
            print "\n" ;
            if (defined($'FILTER)) {
                $filter = $'FILTER ;
                $length = length($'FILTER) ;
                $prefix = ' ' x (9 - $length) ;
                $postfile = ' ' x $length ;
                print STDOUT "$prefix$filter : $postfix" ;
            }
            print STDOUT $message ;
        }
    }
    else {
       print STDOUT $message ;
    }
}
       
  
sub MErr {
    local($message, $type) = @_ ;
  
    # Currently if $type is non-zero then program exits from die.  This
    # could be changed to allow various values for type to specify what
    # type of error message (FATAL, WARNING, and so on).
   
    print "\n" ;
    print STDERR "$'FILTER " if (defined($'FILTER));
    print STDERR "ERROR: $message" ;
    die "Exiting...\n" if (defined($type) && $type) ;
}



1;
